"""

    """

from src.mirutil.jdate import make_datetime_from_iso_jdate_time as mdt


dt1 = '1399-12-12T12:12:12'
mdt(dt1)

##
from src.mirutil.jdate import make_datetime_from_iso_jdate_time as vecmdt


dt2 = '1389-12-12T12:11:21'

vecmdt(dt2)

##
